/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "WorkspaceCreator.h"
#include "MonotoneUtil.h"

#include <QInputDialog>

WorkspaceCreator::WorkspaceCreator(const QString & databaseFile, const QString & rev, const QDir & dir)
  : FileExporter(databaseFile, rev, dir)
{}

bool WorkspaceCreator::run()
{
    //
    // at first determine the branch name(s) of the revision and let the
    // user select one branch of a list of branches
    //
    RevisionCerts certs = MonotoneUtil::getRevisionCerts(databaseFile, revision);

    QStringList branches;
    for (int i=0,j=certs.size(); i<j; i++)
    {
        if (certs.at(i).first == "branch")
            branches.append(certs.at(i).second);
    }

    if (branches.size() == 0)
    {
        // TODO: shall we ask the user here for a new branch name?
        C(QString("Revision '%1' has no branch cert attached").arg(revision));
        return false;
    }

    QString selectedBranch = branches.at(0);

    if (branches.size() > 1)
    {
        bool ok;
        QString branch = QInputDialog::getItem(
            NULL, tr("Select a branch"),
            tr("The selected revision has more than one\n"
               "branch cert attached - please select one:"),
            branches, 0, false, &ok);

        if (!ok || branch.isEmpty())
        {
            D("User aborted checkout");
            return false;
        }
        selectedBranch = branch;
    }

    //
    // now we query the manifest contents
    //
    FileEntryList entries =
        MonotoneUtil::getRevisionManifest(databaseFile, revision);

    //
    // finally before we export anything, we create the _MTN directory layout
    //
    if (!rootDir.mkdir("_MTN"))
    {
         C("Cannot create directory");
         return false;
    }

    I(rootDir.cd("_MTN"));

    // _MTN/revision
    QFile revfile(rootDir.filePath("revision"));
    if (!revfile.open(QIODevice::WriteOnly))
    {
        C("Can't open revision for writing");
        return false;
    }

    QString rev = QString("format_version \"1\"\n\n"
        "new_manifest [0000000000000000000000000000000000000000]\n\n"
        "old_revision [%1]\n").arg(revision);

    I(revfile.write(rev.toLatin1()));
    revfile.close();

    // _MTN/format
    QFile formatfile(rootDir.filePath("format"));
    if (!formatfile.open(QIODevice::WriteOnly))
    {
        C("Can't open format for writing");
        return false;
    }

    I(formatfile.write(QString("2\n").toLatin1()));
    formatfile.close();

    // _MTN/log (initally empty)
    QFile logfile(rootDir.filePath("log"));
    if (!logfile.open(QIODevice::WriteOnly))
    {
        C("Can't open log for writing");
        return false;
    }
    logfile.close();

    // _MTN/options
    QFile optfile(rootDir.filePath("options"));
    if (!optfile.open(QIODevice::WriteOnly))
    {
        C("Can't open revision for writing");
        return false;
    }

    // the keydir entry is not needed and automatically added to the
    // list of options on the first commit
    QString options =
        QString("database \"%1\"\n"
                "branch \"%2\"\n")
        .arg(databaseFile)
        .arg(selectedBranch);

    I(optfile.write(options.toLatin1()));
    optfile.close();

    // change back to the root dir
    I(rootDir.cdUp());

    return FileExporter::run(entries);
}

