/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "MenuBar.h"
#include "Settings.h"
#include "vocab.h"

#include <QApplication>
#include <QFileDialog>
#include <QShortcut>

MenuBar::MenuBar(QWidget * parent) : QMenuBar(parent)
{
    // ensure that the shortcut keys are properly recognized by linguist
    QShortcut::tr("Ctrl");
    QShortcut::tr("Alt");
    QShortcut::tr("Shift");
    QShortcut::tr("Meta");

    //
    // The File menu
    //
    actionOpen_Workspace = new QAction(tr("Open Workspace"), this);
    actionOpen_Workspace->setShortcut(tr("Ctrl+O"));
    actionOpen_Database = new QAction(tr("Open Database"), this);
    actionOpen_Database->setShortcut(tr("Ctrl+Shift+O"));
    actionPreferences = new QAction(tr("Preferences..."), this);
    actionPreferences->setMenuRole(QAction::PreferencesRole);
    actionPreferences->setShortcut(tr("Ctrl+P"));
    actionCheck_for_updates = new QAction(tr("Check for updates"), this);
    actionClose = new QAction(tr("Close"), this);
    actionClose->setShortcut(tr("Ctrl+W"));
    actionQuit = new QAction(tr("Quit"), this);
    actionQuit->setShortcut(tr("Ctrl+Q"));

    menuFile = new QMenu(tr("File"), this);
    menuRecent_Databases = new QMenu(tr("Recent databases..."), menuFile);
    menuRecent_Workspaces = new QMenu(tr("Recent workspaces..."), menuFile);

    menuFile->addAction(actionOpen_Workspace);
    menuFile->addAction(menuRecent_Workspaces->menuAction());
    menuFile->addSeparator();
    menuFile->addAction(actionOpen_Database);
    menuFile->addAction(menuRecent_Databases->menuAction());
    menuFile->addSeparator();
    menuFile->addAction(actionPreferences);
    menuFile->addAction(actionCheck_for_updates);
    menuFile->addSeparator();
    menuFile->addAction(actionClose);
    menuFile->addAction(actionQuit);

    //
    // The Window menu
    //
    actionRaise_all_windows = new QAction(tr("Raise all windows"), this);

    menuWindow = new QMenu(tr("Window"), this);
    menuWindow->addAction(actionRaise_all_windows);
    menuWindow->addSeparator();
    // all opened windows are placed before this named separator -
    // by defining that this separator is invisible, it should not disturb
    // the look of the rendered menu
    actionWindowSeparator = menuWindow->addSeparator();
    actionWindowSeparator->setVisible(false);

    // The Help menu
    //
    actionAbout_Qt = new QAction(tr("About Qt"), this);
    actionAbout_Qt->setMenuRole(QAction::AboutQtRole);
    actionAbout_guitone = new QAction(tr("About guitone"), this);
    actionAbout_guitone->setMenuRole(QAction::AboutRole);

    menuHelp = new QMenu(tr("Help"), this);

    menuHelp->addAction(actionAbout_Qt);
    menuHelp->addAction(actionAbout_guitone);

    // add the created menus to this menu bar
    addAction(menuFile->menuAction());
    addAction(menuWindow->menuAction());
    addAction(menuHelp->menuAction());

    connect(
        actionOpen_Database, SIGNAL(triggered()),
        this, SLOT(openDatabase())
    );

    connect(
        actionOpen_Workspace, SIGNAL(triggered()),
        this, SLOT(openWorkspace())
    );

    connect(
        actionQuit, SIGNAL(triggered()),
        qApp, SLOT(quit())
    );

    connect(
        actionAbout_Qt, SIGNAL(triggered()),
        qApp, SLOT(aboutQt())
    );

    connect(
        actionAbout_guitone, SIGNAL(triggered()),
        this, SIGNAL(showAbout())
    );

    connect(
        actionPreferences, SIGNAL(triggered()),
        this, SIGNAL(showPreferences())
    );

    connect(
        actionCheck_for_updates, SIGNAL(triggered()),
        this, SIGNAL(checkForApplicationUpdates())
    );

    connect(
        actionClose, SIGNAL(triggered()),
        this, SIGNAL(closeWindow())
    );

    connect(
        actionRaise_all_windows, SIGNAL(triggered()),
        this, SIGNAL(raiseAllWindows())
    );
}

MenuBar::~MenuBar() {}

void MenuBar::updateRecentLists()
{
    menuRecent_Workspaces->clear();
    menuRecent_Databases->clear();

    QStringList previousWs = Settings::getItemList("RecentWorkspaceList");
    int elemCount = previousWs.size();
    if (elemCount == 0)
    {
        menuRecent_Workspaces->addAction(tr("No previous workspaces available."));
    }
    else
    {
        QAction * act;
        for (int i = 0; i < elemCount; ++i)
        {
            act = menuRecent_Workspaces->addAction(
                tr("&%1 %2").arg(i + 1).arg(previousWs[i]),
                this,
                SLOT(openRecentWorkspace())
            );
            act->setData(previousWs[i]);
        }
    }

    QStringList previousDb = Settings::getItemList("RecentDatabaseList");
    elemCount = previousDb.size();
    if (elemCount == 0)
    {
        menuRecent_Databases->addAction(tr("No previous databases available."));
    }
    else
    {
        QAction * act;
        for (int i = 0; i < elemCount; ++i)
        {
            act = menuRecent_Databases->addAction(
                tr("&%1 %2").arg(i + 1).arg(previousDb[i]),
                this,
                SLOT(openRecentDatabase())
            );
            act->setData(previousDb[i]);
        }
    }
}

void MenuBar::openDatabase()
{
    QString fn = QFileDialog::getOpenFileName(
        0,
        tr("Select your database..."),
        QString(),
        tr("monotone Databases (*.mtn *.db)")
    );

    if (fn.isEmpty()) return;

    emit loadDatabase(fn);
}

void MenuBar::openWorkspace()
{
    QString fn = QFileDialog::getExistingDirectory(0,
        tr("Select your workspace...")
    );

    if (fn.isEmpty()) return;

    emit loadWorkspace(fn);
}

void MenuBar::openRecentWorkspace()
{
    QAction * action = qobject_cast<QAction *>(sender());
    emit loadWorkspace(action->data().toString());
}

void MenuBar::openRecentDatabase()
{
    QAction * action = qobject_cast<QAction *>(sender());
    emit loadDatabase(action->data().toString());
}

void MenuBar::updateWindowList(const QStringList & windowTitles)
{
    // remove old actions
    QList<QAction*> actions = menuWindow->actions();
    for (int i=0, j=actions.size(); i<j; i++)
    {
        QAction * act = actions.at(i);
        if (act->data().isValid())
        {
            disconnect(
                act, SIGNAL(triggered()),
                this, SLOT(activateOtherWindow())
            );
            menuWindow->removeAction(act);
        }
    }

    // TODO: we should integrate a small indicator which window is actually
    // active here, but this needs central work in the App
    for (int i=0, j=windowTitles.size(); i<j; i++)
    {
        QAction * act = new QAction(tr("&%1 %2").arg(i + 1).arg(windowTitles.at(i)), this);
        act->setData(i);
        connect(
            act, SIGNAL(triggered()),
            this, SLOT(activateOtherWindow())
        );
        menuWindow->insertAction(actionWindowSeparator, act);

    }
}

void MenuBar::activateOtherWindow()
{
    // FIXME: the list of windows is not constant, theoretically it might
    // be possible that we've just been notified to raise a window which
    // was just removed. Since we can't / won't transport pointers around
    // we try to identify the window by its index in the windowList again,
    // if this fails, we do nothing. It might occur that we raise the wrong
    // window by this, of course, but all this is better than the alternatives.
    QAction * action = qobject_cast<QAction *>(sender());
    if (action)
    {
        emit activateWindow(action->data().toInt());
    }
}

