/***************************************************************************
 *   Copyright (C) 2006 by Ingo Maindorfer                                 *
 *   ingo@liquidcooling.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "KeyManagement.h"
#include "GenerateKeypair.h"
#include "MonotoneManager.h"
#include "Settings.h"

#include <QMessageBox>
#include <QApplication>
#include <QClipboard>

KeyManagement::KeyManagement(QWidget * parent, const DatabaseFile & db)
  : Dialog(parent), databaseFile(db)
{
    setupUi(this);
    Dialog::init();

    model = new Keys(this, databaseFile);
    keyList->setModel(model);

    connect(
        generateKey, SIGNAL(clicked()),
        this, SIGNAL(generateKeypair())
    );

    connect(
        keyList, SIGNAL(contextMenuRequested(const QModelIndexList &, const QPoint &)),
        this, SLOT(contextMenuEvent(const QModelIndexList &, const QPoint &))
    );

    popupMenu = new QMenu(this);

    QAction * act = new QAction(tr("Copy key name to clipboard"), this);
    connect(act, SIGNAL(triggered()), this, SLOT(copyKeyNameToClipboard()));
    popupMenu->addAction(act);

    act = new QAction(tr("Copy public key hash to clipboard"), this);
    connect(act, SIGNAL(triggered()), this, SLOT(copyPubkeyHashToClipboard()));
    popupMenu->addAction(act);

    act = new QAction(tr("Copy public key data to clipboard"), this);
    connect(act, SIGNAL(triggered()), this, SLOT(copyPubkeyDataToClipboard()));
    popupMenu->addAction(act);
}

KeyManagement::~KeyManagement()
{
    delete model;
    qDeleteAll(popupMenu->findChildren<QAction*>());
    delete popupMenu;
}

void KeyManagement::readKeys()
{
    model->readKeys();
}

void KeyManagement::contextMenuEvent(const QModelIndexList & indexList, const QPoint & p)
{
    Q_UNUSED(indexList);
    popupMenu->exec(p);
}

Key * KeyManagement::getKeyFromSelection() const
{
    QItemSelectionModel * selection = keyList->selectionModel();
    QList<QModelIndex> indexList = selection->selectedIndexes();
    // we're only interested in the first column's internal pointer
    return static_cast<Key *>(indexList[0].internalPointer());
}

void KeyManagement::copyKeyNameToClipboard()
{
    Key * key = getKeyFromSelection();
    QClipboard * clipboard = QApplication::clipboard();
    clipboard->setText(key->name);
}

void KeyManagement::copyPubkeyHashToClipboard()
{
    Key * key = getKeyFromSelection();
    QClipboard * clipboard = QApplication::clipboard();
    clipboard->setText(key->public_hash);
}

//! \todo should be converted to automate pubkey, if available
void KeyManagement::copyPubkeyDataToClipboard()
{
    Key * key = getKeyFromSelection();
    QClipboard * clipboard = QApplication::clipboard();

    QStringList args;
    QByteArray output;
    args << "-d" << databaseFile << "pubkey" << key->name;

    if (!MonotoneManager::singleRun(Settings::getMtnBinaryPath(), args, output))
    {
        QMessageBox::critical(
            this,
            tr("Couldn't query public key data"),
            tr("Failed to query public key data for key \"%1\".")
                .arg(key->name),
            QMessageBox::Ok
         );
         return;
    }

    clipboard->setText(QString::fromUtf8(output));
}

