/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "WorkspaceDialogManager.h"
#include "MonotoneUtil.h"
#include "GuitoneCore.h"

WorkspaceDialogManager::WorkspaceDialogManager(QWidget * parent)
  : DatabaseDialogManager(parent), commitRevision(0),
    unaccountedRenames(0), updateWorkspace(0) {}

WorkspaceDialogManager::~WorkspaceDialogManager()
{
    cleanup();
}

void WorkspaceDialogManager::cleanup()
{
    if (commitRevision)     { delete commitRevision; commitRevision = 0; }
    if (unaccountedRenames) { delete unaccountedRenames; unaccountedRenames = 0; }
    if (updateWorkspace)    { delete updateWorkspace; updateWorkspace = 0; }
}

void WorkspaceDialogManager::closeAllDialogs()
{
    if (commitRevision)     commitRevision->close();
    if (unaccountedRenames) unaccountedRenames->close();
    if (updateWorkspace)    updateWorkspace->close();

    DatabaseDialogManager::closeAllDialogs();
}

void WorkspaceDialogManager::init(const WorkspacePath & path)
{
    DatabaseDialogManager::init(APP->manager()->getDatabaseFilePath(path));
    workspacePath = path;
    cleanup();
}


void WorkspaceDialogManager::showCommitRevision()
{
    showCommitRevision(QStringList());
}

void WorkspaceDialogManager::showCommitRevision(const QStringList & paths)
{
    if (!commitRevision)
    {
        commitRevision = new CommitRevision(parentWidget(), workspacePath);

        connect(
            commitRevision, SIGNAL(revisionCommitted(const QString &)),
            this, SIGNAL(revisionCommitted(const QString &))
        );

        connect(
            commitRevision, SIGNAL(diffFile(const QString &)),
            this,  SLOT(showFileDiff(const QString &))
        );
    }

    commitRevision->readWorkspaceRevision(paths);
    showDialog(commitRevision);
}

void WorkspaceDialogManager::showFileDiff(const QString & file)
{
    showFileDiff(file, QString(), QString());
}

void WorkspaceDialogManager::showFileDiff(const QString & file, const QString & base, const QString & target)
{
    if (!fileDiff)
    {
        fileDiff = new FileDiff(parentWidget());
    }

    fileDiff->forWorkspace(workspacePath, file, base, target);
    showDialog(fileDiff);
}

void WorkspaceDialogManager::showFileHistory(const QString & file)
{
    DatabaseDialogManager::showFileHistory(file,
        MonotoneUtil::getBaseWorkspaceRevision(workspacePath));
}

void WorkspaceDialogManager::showRevisionDiff(const QString & file, const QString & base, const QString & target)
{
    if (!revisionDiff)
    {
        revisionDiff = new RevisionDiff(parentWidget());

        connect(
            revisionDiff, SIGNAL(fileDiff(const QString &, const QString &, const QString &)),
            this, SLOT(showFileDiff(const QString &, const QString &, const QString &))
        );
    }

    revisionDiff->forWorkspace(workspacePath, file, base, target);
    showDialog(revisionDiff);
}

void WorkspaceDialogManager::checkForUnaccountedRenames()
{
    if (!unaccountedRenames)
    {
        unaccountedRenames = new UnaccountedRenames(parentWidget(), workspacePath);
    }

    unaccountedRenames->checkForUnaccountedRenames();
}

void WorkspaceDialogManager::showUpdateWorkspace()
{
    if (!updateWorkspace)
    {
        updateWorkspace = new UpdateWorkspace(parentWidget(), workspacePath);

        connect(
            updateWorkspace, SIGNAL(selectRevisionFromSelector(const QString &)),
            this, SLOT(showSelectRevision(const QString &))
        );

        if (selectRevision)
        {
            connect(
                selectRevision, SIGNAL(revisionSelected(const QString &)),
                updateWorkspace, SLOT(setSelectedRevision(const QString &))
            );
        }
    }

    showDialog(updateWorkspace);
}

